<?php

declare(strict_types=1);

namespace Drupal\lms\Plugin\views\field;

use Drupal\Component\Render\MarkupInterface;
use Drupal\Core\Link;
use Drupal\lms\Entity\CourseStatus as CourseStatusEntity;
use Drupal\views\Attribute\ViewsField;
use Drupal\views\Plugin\views\field\FieldPluginBase;
use Drupal\views\ResultRow;

/**
 * LMS Course user status field.
 */
#[ViewsField(id: 'lms_student_course_status')]
final class CourseStatus extends FieldPluginBase {

  /**
   * {@inheritdoc}
   */
  public function render(ResultRow $values): MarkupInterface {
    $status = $this->getValue($values);
    if ($status === NULL) {
      return $this->t('Not started');
    }

    $status_text = CourseStatusEntity::getStatusText($status);
    $status_class = \strtr($status, ' ', '-');

    $course_status = $values->_relationship_entities['course_status'] ?? NULL;
    if ($course_status === NULL) {
      $course_status = $values->_entity;
    }
    if (!$course_status instanceof CourseStatusEntity) {
      $renderable = [
        '#type' => 'html_tag',
        '#tag' => 'p',
        '#attributes' => [
          'class' => [$status_class],
        ],
        'text' => [
          '#markup' => $status_text,
        ],
      ];
      return $this->getRenderer()->render($renderable);
    }

    // Output as link to results.
    $renderable = Link::createFromRoute($status_text, 'lms.group.results', [
      'group' => $course_status->getCourseId(),
      'user' => $course_status->getUserId(),
    ], [
      'attributes' => [
        '_target' => 'blank',
        'class' => [$status_class],
      ],
    ])->toRenderable();
    return $this->getRenderer()->render($renderable);
  }

}
